from bsition.backend.mongo.utils import get_database
from bsition.backend.postgres import tables as postgres


def create_table(table):
    dbname = get_database()
    docs_coll = dbname["tables"]
    docs_coll.insert_one(table)


def insert_columns(id, name, type, data):
    dbname = get_database()
    docs_coll = dbname["tables"]
    doc = docs_coll.find_one({"_id": id}, {"_id": 0})
    new_column_names = doc["column_names"].copy()
    new_column_names.append(name)
    new_types = doc["types"].copy()
    new_types.append(type)
    docs_coll.update_one({"_id": id}, {"$set": {
        "column_names": new_column_names, "types": new_types}
    })


def edit_column(row_number, column, data, id):
    dbname = get_database()
    docs_coll = dbname["tables"]
    doc = docs_coll.find_one({"_id": id}, {"_id": 0})
    column_names = doc["column_names"]
    new_data = doc["data"].copy()

    i = column_names.index(column)
    new_data[row_number]["column_data"][i] = data

    docs_coll.update_one({"_id": id}, {"$set": {
        "data": new_data}
    })


def sort(id):
    dbname = get_database()
    docs_coll = dbname["tables"]
    doc = docs_coll.find_one({"_id": id}, {"_id": 0})
    column_names = doc["column_names"]

    sort_values = postgres.get_sort(id)
    obj = {}
    for sort in sort_values:
        obj['data.column_data.' + str(column_names.index(sort[1]))] = 1 if sort[2] == 'ASC' else -1

    pipeline = [
        {"$match": {"_id": id}},
        {"$unwind": '$data'},
        {"$sort": obj},
        {"$group": {"_id": '$_id', 'aux': {"$push": '$data'}}},
        {"$project": {'data': '$aux'}}
    ]
    return list(docs_coll.aggregate(pipeline))


def filter(id):
    dbname = get_database()
    docs_coll = dbname["tables"]
    doc = docs_coll.find_one({"_id": id}, {"_id": 0})
    column_names = doc["column_names"]

    filter_values = postgres.get_filter(id)
    obj = {}
    for filter in filter_values:
        match filter[3]:
            case "e":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$eq": filter[2]
                }
            case "ne":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$ne": filter[2]
                }
            case "le":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$lte": filter[2]
                }
            case "ge":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$gte": filter[2]
                }
            case "l":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$lt": filter[2]
                }
            case "g":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$gt": filter[2]
                }
            case "c":
                obj['data.column_data.' + str(column_names.index(filter[1]))] = {
                    "$regex": ".*" + filter[2] + ".*", "$options": "i"
                }
            case "_":
                raise "Invalid filter function"

    pipeline = [
        {"$match": {"_id": id}},
        {"$unwind": '$data'},
        {"$match": obj},
        {"$group": {"_id": '$_id', 'aux': {"$push": '$data'}}},
        {"$project": {'data': '$aux'}}
    ]
    return list(docs_coll.aggregate(pipeline))


def remove_column(id, column):
    dbname = get_database()
    docs_coll = dbname["tables"]
    doc = docs_coll.find_one({"_id": id}, {"_id": 0})
    column_names = doc["column_names"]
    types = doc["types"]
    idx = column_names.index(column)
    del column_names[idx]
    del types[idx]
    data = doc["data"]
    for row in data:
        del row["column_data"][idx]

    docs_coll.update_one({"_id": id}, {"$set": {
        "column_names": column_names, "types": types, "data": data}
    })
